@@ -1,3 +1,24 @@
+/*
+ * This File is part of Davix, The IO library for HTTP based protocols
+ * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+*/
+
+
 #pragma once
 #ifndef DAVIX_REQUESTPARAMS_HPP
 #define DAVIX_REQUESTPARAMS_HPP
@@ -5,7 +26,7 @@
 #include <vector>
 #include <string>
 
- 
+#include <utils/davix_uri.hpp>
 #include <auth/davixauth.hpp>
 
 
@@ -41,6 +62,19 @@ namespace RequestProtocol{
     };
 }
 
+namespace MetalinkMode{
+    enum MetalinkMode{
+        // default mode = failover
+        Auto=0,
+        // Disable Metalink Support
+        Disable,
+        // Use only for failover purpose
+        FailOver,
+        // Enable parallel DL
+        XStream
+    };
+}
+
 
 struct RequestParamsInternal;
 ///
@@ -174,24 +208,56 @@ public:
     /// get the current value of the request protocol
     RequestProtocol::Protocol getProtocol() const;
 
+    /// Enable or disable the usage of the Metalink
+    ///  (RFC-5854 and RFC-6249) with libdavix
+    /// Metalink can be used for fail-over purpose, or multi-source download
+    void setMetalinkMode( const MetalinkMode::MetalinkMode mode);
+
+    /// get the Current Metalink mode
+    const MetalinkMode::MetalinkMode getMetalinkMode() const;
+
     /// set the keep alive value of the associated session
     void setKeepAlive(const bool keep_alive_flag);
 
     /// get the keep alive value of this request params
     bool getKeepAlive() const;
 
+
+    /// Add a custom header line that has to be included in the requests
+    ///  @param key key of the header
+    ///  @param val value of the header
+    void addHeader(const std::string &key, const std::string &val);
+
+    /// return the list of custom headers configured
+    const HeaderVec & getHeaders() const;
+
+    /// set a SOCKS5 proxy server for intermediate usage
+    /// example: setProxyServer("socks5://login:password@socks5.exmaple.org:8080")
+    /// @param proxy_url url of the proxy server
+    void setProxyServer(const Uri & proxy_url);
+
+    /// get current SOCKS5 proxy server
+    /// @return URL of the server or NULL if not defined
+    const Uri* getProxyServer() const;
+
     /// internal usage
     void* getParmState() const;
 
 
+    /// swap two RequestParams content
+    /// fast operation
+    void swap(RequestParams & params);
+
 private:
 
    // dptr
     RequestParamsInternal* d_ptr;
 
+
 };
 
 
+
 } // namespace Davix
 
 #endif // DAVIX_REQUESTPARAMS_HPP
