@@ -1,13 +1,33 @@
+/*
+ * This File is part of Davix, The IO library for HTTP based protocols
+ * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+*/
+
+
 #ifndef DAVIX_HTTPREQUEST_H
 #define DAVIX_HTTPREQUEST_H
 
 #include <vector>
 #include <unistd.h>
 #include <utils/davix_types.hpp>
-#include <davixuri.hpp>
+#include <utils/davix_uri.hpp>
 #include <status/davixstatusrequest.hpp>
 #include <params/davixrequestparams.hpp>
-#include <request/httpcachetoken.hpp>
 
 #ifndef __DAVIX_INSIDE__
 #error "Only davix.h or davix.hpp should be included."
@@ -41,6 +61,7 @@ typedef dav_ssize_t (*HttpBodyProvider)(void *userdata,
 
 class NEONRequest;
 class NEONSessionFactory;
+class HttpCacheToken;
 
 namespace RequestFlag{
     ///
@@ -54,7 +75,6 @@ namespace RequestFlag{
 
 }
 
-
 /// @class HttpRequest
 /// @brief Http low level request interface
 /// HTTPRequest is the main davix class for low level HTTP queries
@@ -246,19 +266,18 @@ public:
     /// @return true if this header exist or false if it does not
     bool getAnswerHeader(const std::string & header_name, std::string & value) const;
 
-    ///
-    /// @brief Extract a cache token from this Request
-    ///
-    /// a cache token allows to optimize future request
-    /// cache tokens are dynamically allocated and need to be free
-    /// @return cache token or null pointer if not available
-    HttpCacheToken* extractCacheToken() const;
 
     ///
-    /// \brief use the cache token of a previous request, enable request o
-    ///  optimizations ( session re-use, redirection caching, server operation support )
-    /// \param token
-    ///
+    /// get all the headers associated with this answer
+    /// @param value : vector of headers
+    /// @return true if this header exist or false if it does not
+    size_t getAnswerHeaders( HeaderVec & vec_headers) const;
+
+
+    /// @deprecated not in use anymore
+    HttpCacheToken* extractCacheToken() const;
+
+    /// @deprecated not in use anymore
     void useCacheToken(const HttpCacheToken* token);
 
     /// set a HttpRequest flag
@@ -270,6 +289,9 @@ public:
 private:
     NEONRequest* d_ptr;
 
+    void runPreRunHook();
+    void runRegisterHooks();
+
     friend class NEONRequest;
     friend class NEONSessionFactory;
 
