@@ -1,5 +1,5 @@
+#include <davix_internal.hpp>
 #include <auth/davixauth.hpp>
-#include <sstream>
 #include <base64/base64.hpp>
 #include <hmac_sha1/hmacsha1.hpp>
 
@@ -12,4 +12,15 @@ std::string getAwsAuthorizationField(const std::string & stringToSign, const std
     return ss.str();
 }
 
+std::vector<std::string> v;
+
+SessionInfo::SessionInfo() : data(NULL){
+
+}
+
+std::vector<std::string> & SessionInfo::getReadableDN() const{
+    // TODO: implement proper DN mapping
+    return v;
+}
+
 }
