@@ -1,16 +1,31 @@
-#include <config.h>
+/*
+ * This File is part of Davix, The IO library for HTTP based protocols
+ * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+*/
+
+#include <davix_internal.hpp>
 #include "httpiovec.hpp"
-#include <cstring>
-#include <cstddef>
-#include <logger/davix_logger_internal.h>
+#include <utils/davix_logger_internal.hpp>
 #include <string_utils/stringutils.hpp>
-#include <deque>
-#include <functional>
-#include <algorithm>
-#include <sstream>
 
 #include <map>
 
+using namespace StrUtil;
 
 // remove trailing crlf
 template<class InputIterator>
@@ -33,7 +48,7 @@ template<class InputIterator>
 namespace Davix{
 
 const std::string HttpIoVec_scope(){
-    return "Davix::HttpVecOps";
+    return "Davix::HttpIOVecOps";
 }
 
 
@@ -71,26 +86,34 @@ int davIOVecProvider(const DavIOVecInput *input_vec, dav_ssize_t & counter, dav_
     return -1;
 }
 
-dav_ssize_t HttpVecOps::readPartialBufferVec(const DavIOVecInput * input_vec,
+dav_ssize_t HttpIOVecOps::preadVec(IOChainContext & iocontext, const DavIOVecInput * input_vec,
                           DavIOVecOuput * output_vec,
-                          const dav_size_t count_vec, DavixError** err){
+                          const dav_size_t count_vec){
 
     if(count_vec ==0)
         return 0;
 
+    if(count_vec ==1){ // one offset read request, no need of multi part
+            const dav_ssize_t res= _start->pread(iocontext, input_vec->diov_buffer, input_vec->diov_size, input_vec->diov_offset);
+            output_vec->diov_buffer = input_vec->diov_buffer;
+            output_vec->diov_size= res;
+            return res;
+    }
+
+
 
     DavixError * tmp_err=NULL;
     dav_ssize_t tmp_ret=-1, ret = 0;
     ptrdiff_t p_diff=0;
     dav_ssize_t counter = 0;
     // determine the Range header request in order to determine the number of request
-    stdx11::function<int (dav_off_t &, dav_off_t &)> offsetProvider( stdx11::bind(davIOVecProvider, input_vec, counter, (dav_ssize_t) count_vec,
-                         stdx11::placeholders::_1, stdx11::placeholders::_2));
+    boost::function<int (dav_off_t &, dav_off_t &)> offsetProvider( boost::bind(davIOVecProvider, input_vec, counter, (dav_ssize_t) count_vec,
+                         _1, _2));
 
     // header line need to be inferior to 8K on Apache2 / Ngix
     // in Addition, some S3 implementation limit the total header size to 8k....
     // 7900 bytes maximum for the range seems to be a ood compromise
-    std::vector< std::pair<dav_size_t, std::string> > vecRanges = generateRangeHeaders(7900, offsetProvider);
+    std::vector< std::pair<dav_size_t, std::string> > vecRanges = generateRangeHeaders(3900, offsetProvider);
 
 
     DAVIX_DEBUG(" -> getPartialVec operation for %d vectors", count_vec);
@@ -99,23 +122,27 @@ dav_ssize_t HttpVecOps::readPartialBufferVec(const DavIOVecInput * input_vec,
         DAVIX_DEBUG(" -> getPartialVec request for %ld chunks", it->first);
 
         if(it->first == 1){ // one chunk only : no need of multi part
-            if ( (tmp_ret = _io.readPartialBuffer(
-                      (input_vec + p_diff)->diov_buffer,
-                      (input_vec+p_diff)->diov_size,
-                      (input_vec+p_diff)->diov_offset, &tmp_err)) < 0){
-                ret = -1;
-                break;
-            }
+            TRY_DAVIX{
+                if ( (tmp_ret = _start->pread(
+                          iocontext,
+                          (input_vec + p_diff)->diov_buffer,
+                          (input_vec+p_diff)->diov_size,
+                          (input_vec+p_diff)->diov_offset)) < 0){
+                    ret = -1;
+                    break;
+                }
+            }CATCH_DAVIX(&tmp_err);
+
             (output_vec+ p_diff)->diov_size = ret;
             (output_vec+ p_diff)->diov_buffer = (input_vec + p_diff)->diov_buffer;
             p_diff += 1;
             ret += tmp_ret;
 
         }else{
-            GetRequest req (_c,_url, &tmp_err);
+            GetRequest req (iocontext._context, iocontext._uri, &tmp_err);
             if(tmp_err == NULL){
-                req.setParameters(_params);
-                req.useCacheToken(&_cacheToken);
+                RequestParams request_params(iocontext._reqparams);
+                req.setParameters(request_params);
                 req.addHeaderField(req_header_byte_range, it->second);
                 if( (tmp_ret = readPartialBufferVecRequest(req, input_vec+ p_diff, output_vec+ p_diff, it->first, &tmp_err)) <0){
                     ret = -1;
@@ -133,12 +160,11 @@ dav_ssize_t HttpVecOps::readPartialBufferVec(const DavIOVecInput * input_vec,
 
 
     DAVIX_DEBUG(" <- getPartialVec operation for %d vectors", count_vec);
-    if(tmp_err)
-        DavixError::propagateError(err, tmp_err);
+    checkDavixError(&tmp_err);
     return ret;
 }
 
-dav_ssize_t HttpVecOps::readPartialBufferVecRequest(HttpRequest & _req,
+dav_ssize_t HttpIOVecOps::readPartialBufferVecRequest(HttpRequest & _req,
                           const DavIOVecInput * input_vec,
                           DavIOVecOuput * output_vec,
                           const dav_size_t count_vec, DavixError** err){
@@ -159,8 +185,8 @@ dav_ssize_t HttpVecOps::readPartialBufferVecRequest(HttpRequest & _req,
                  httpcodeToDavixCode(_req.getRequestCode(),davix_scope_http_request(),", ", &tmp_err);
         }
     }
-    if(tmp_err)
-        DavixError::propagateError(err, tmp_err);
+
+    DavixError::propagateError(err, tmp_err);
     DAVIX_TRACE(" <- Davix Vector operation");
     return ret;
 }
@@ -170,11 +196,11 @@ int http_extract_boundary_from_content_type(const std::string & buffer, std::str
     dav_size_t pos_bound;
     static const std::string delimiter = "\";";
     if( (pos_bound= buffer.find(ans_header_boundary_field)) != std::string::npos){
-        std::vector<std::string> tokens = stringTokSplit(buffer.substr(pos_bound + ans_header_boundary_field.size()), delimiter);
+        std::vector<std::string> tokens = tokenSplit(buffer.substr(pos_bound + ans_header_boundary_field.size()), delimiter);
         if( tokens.size() >= 1
             && tokens[0].size() > 0
             && tokens[0].size() <= 70){
-            DAVIX_TRACE("Multi part content type : %s", boundary.c_str());
+            DAVIX_TRACE("Multi part boundary: %s", boundary.c_str());
             std::swap(boundary,tokens[0]);
             return 0;
         }
@@ -209,11 +235,11 @@ int find_header_params(char* buffer, dav_size_t buffer_len, dav_size_t* part_siz
     if(p == NULL)
         return -1;
     std::string header_type(buffer, p - buffer);
-    if( string_compare_ncase(ans_header_byte_range, 0, p - buffer, buffer) !=0) // check header type
+    if( compare_ncase(ans_header_byte_range, 0, p - buffer, buffer) !=0) // check header type
         return 0;
 
-    std::vector<std::string> tokens = stringTokSplit(std::string(p+1),delimiter);     // parse header
-    if(tokens.size() != 3)
+    std::vector<std::string> tokens = tokenSplit(std::string(p+1),delimiter);     // parse header
+    if(tokens.size() < 2)
         return -1;
 
     long chunk_size[2];
@@ -304,7 +330,7 @@ dav_ssize_t copyChunk(HttpRequest & req, const DavIOVecInput *i,
 }
 
 
-dav_ssize_t HttpVecOps::parseMultipartRequest(HttpRequest & _req,
+dav_ssize_t HttpIOVecOps::parseMultipartRequest(HttpRequest & _req,
                                             const DavIOVecInput *input_vec,
                                             DavIOVecOuput * output_vec,
                                             const dav_size_t count_vec, DavixError** err){
@@ -321,7 +347,7 @@ dav_ssize_t HttpVecOps::parseMultipartRequest(HttpRequest & _req,
     DAVIX_DEBUG("Davix::parseMultipartRequest multi-part boundary %s", boundary.c_str());
 
     while(off < count_vec){
-       DAVIX_DEBUG("Davix::parseMultipartRequest try to find chunk offset:%ld size%ld", input_vec[off].diov_offset, input_vec[off].diov_size);
+       DAVIX_DEBUG("Davix::parseMultipartRequest try to find chunk offset:%ld size %ld", input_vec[off].diov_offset, input_vec[off].diov_size);
        ChunkInfo infos;
        int n_try = 0;
        if( (tmp_ret = parse_multi_part_header(_req, boundary, infos,
@@ -350,7 +376,7 @@ dav_ssize_t HttpVecOps::parseMultipartRequest(HttpRequest & _req,
        off++;
     }
 
-    DAVIX_TRACE("Davix::parseMultipartRequest end");
+    DAVIX_TRACE("Davix::parseMultipartRequest end %d %d", off, count_vec);
     return ret;
 }
 
@@ -460,7 +486,7 @@ static dav_ssize_t sum_all_chunk_size(const MapChunk & cmap){
     return res;
 }
 
-dav_ssize_t HttpVecOps::simulateMultiPartRequest(HttpRequest & _req, const DavIOVecInput *input_vec,
+dav_ssize_t HttpIOVecOps::simulateMultiPartRequest(HttpRequest & _req, const DavIOVecInput *input_vec,
                                  DavIOVecOuput * output_vec,
                    const dav_size_t count_vec, DavixError** err){
     DAVIX_TRACE(" -> Davix vec : 200 full file, simulate vec io");
@@ -471,7 +497,7 @@ dav_ssize_t HttpVecOps::simulateMultiPartRequest(HttpRequest & _req, const DavIO
     fill_map_chunk(cmap, input_vec, output_vec, count_vec);
     MapChunk::iterator it_start=cmap.begin(),it_end = cmap.begin();
     while( (tmp_read_size = _req.readBlock(buffer, DAVIX_READ_BLOCK_SIZE, err)) >0){
-        balance_iterator_windows(cmap, it_start, it_end, total_read_size, tmp_read_size); // re-balance the itnerested windows
+        balance_iterator_windows(cmap, it_start, it_end, total_read_size, tmp_read_size); // re-balance the interested windows
         fill_concerned_chunk_buffer(cmap, it_start, it_end, buffer, tmp_read_size, total_read_size); // fill the interested window
         total_read_size += tmp_read_size;
     }
