@@ -1,23 +1,61 @@
+/*
+ * This File is part of Davix, The IO library for HTTP based protocols
+ * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+*/
+
+#include <davix_internal.hpp>
 #include "neonrequest.hpp"
 
-#include <config.h>
-#include <logger/davix_logger_internal.h>
+#include <utils/davix_logger_internal.hpp>
 #include <libs/time_utils.h>
 #include <ne_redirect.h>
 #include <ne_request.h>
+#include <davix_context_internal.hpp>
 #include <neon/neonsession.hpp>
 #include <fileops/fileutils.hpp>
-#include <request/httpcachetoken_internal.hpp>
-#include <memory>
-#include <algorithm>
-#include <iterator>
-#include <sstream>
-#include <cstring>
-#include <limits>
+
+
 
 
 namespace Davix {
 
+class NEONSessionExtended : public NEONSession{
+public:
+    NEONSessionExtended(NEONRequest* r, const Uri &uri, const RequestParams &p, DavixError **err)
+        : NEONSession(r->_c, uri, p, err), _r(r)
+    {
+        if(get_ne_sess() != NULL){
+            ne_hook_pre_send(get_ne_sess(), NEONRequest::neon_hook_pre_send, (void*)r);
+            ne_hook_post_headers(get_ne_sess(), NEONRequest::neon_hook_pre_rec, (void*) r);
+        }
+    }
+
+    virtual ~NEONSessionExtended(){
+        if(get_ne_sess() != NULL){
+            ne_unhook_pre_send(get_ne_sess(), NEONRequest::neon_hook_pre_send, (void*)_r);
+            ne_unhook_post_headers(get_ne_sess(), NEONRequest::neon_hook_pre_rec, (void*) _r);
+        }
+    }
+
+private:
+    NEONRequest* _r;
+};
+
 
 
 static void neonrequest_eradicate_session(NEONSession& sess, DavixError ** err){
@@ -39,11 +77,11 @@ void neon_generic_error_mapper(int ne_status, StatusCode::Code & code, std::stri
              str= "Domain name resolution failed";
              break;
         case NE_AUTH:
-            code = StatusCode::AuthentificationError;
+            code = StatusCode::AuthenticationError;
             str=  "Authentification failed on server";
             break;
         case NE_PROXYAUTH:
-            code = StatusCode::AuthentificationError;
+            code = StatusCode::AuthenticationError;
             str=  "Authentification failed on proxy";
             break;
         case NE_CONNECT:
@@ -75,8 +113,11 @@ void neon_to_davix_code(int ne_status, ne_session* sess, const std::string & sco
     std::string str;
     switch(ne_status){
         case NE_ERROR:
-             str = std::string("Neon error : ").append(ne_get_error(sess));
-             code = StatusCode::ConnectionProblem;
+             str = std::string("Neon error: ").append(ne_get_error(sess));
+             if (str.find("SSL handshake failed") == std::string::npos)
+                 code = StatusCode::ConnectionProblem;
+             else
+                 code = StatusCode::SSLError;
              break;
         default:
             neon_generic_error_mapper(ne_status, code, str);
@@ -108,14 +149,15 @@ void neon_simple_req_code_to_davix_code(int ne_status, ne_session* sess, const s
 }
 
 
-NEONRequest::NEONRequest(NEONSessionFactory& f, const Uri & uri_req) :
+
+NEONRequest::NEONRequest(HttpRequest & h, Context& context, const Uri & uri_req) :
     _req_flag(RequestFlag::IdempotentRequest),
     params(),
-    _cache_info(),
     _neon_sess(),
     _req(NULL),
-    _current(uri_req),
-    _orig(uri_req),
+    _current( new Uri(uri_req)),
+    _orig(_current),
+    _number_try(0),
     _last_read(-1),
     _vec(),
     _vec_line(),
@@ -127,7 +169,8 @@ NEONRequest::NEONRequest(NEONSessionFactory& f, const Uri & uri_req) :
     _content_provider(),
     _ans_size(-1),
     _request_type("GET"),
-    _f(f),
+    _h(h),
+    _c(context),
     req_started(false),
     req_running(false),
     _last_request_flag(0),
@@ -149,7 +192,9 @@ NEONRequest::~NEONRequest(){
 
 
 void NEONRequest::reqReset(){
-    if(req_running) endRequest(NULL);
+    if(req_running) {
+        endRequest(NULL);
+    }
     free_request();
 }
 
@@ -158,10 +203,15 @@ int NEONRequest::create_req(DavixError** err){
        reqReset();
     }
 
+    boost::shared_ptr<Uri> redir_url = ContextExplorer::SessionFactoryFromContext(_c).redirectionResolve(_request_type, *_current);
+    if(redir_url.get() != NULL){
+        _current.swap(redir_url);
+    }
+
     if( pick_sess(err) < 0)
         return -1;
 
-    _req= ne_request_create(_neon_sess->get_ne_sess(), _request_type.c_str(), _current.getPathAndQuery().c_str());
+    _req= ne_request_create(_neon_sess->get_ne_sess(), _request_type.c_str(), _current->getPathAndQuery().c_str());
     configure_req();
 
     return 0;
@@ -169,7 +219,7 @@ int NEONRequest::create_req(DavixError** err){
 
 int NEONRequest::pick_sess(DavixError** err){
     DavixError * tmp_err=NULL;
-    _neon_sess.reset(new NEONSession(_f, _current, params, &tmp_err) );
+    _neon_sess.reset(static_cast<NEONSession*>(new NEONSessionExtended(this, *_current, params, &tmp_err)));
     if(tmp_err){
         _neon_sess.reset(NULL);
         DavixError::propagateError(err, tmp_err);
@@ -209,9 +259,20 @@ void NEONRequest::configure_req(){
         ne_set_request_body_buffer(_req, _content_ptr, _content_len);       
     }
 
+    const HeaderVec &  h  = params.getHeaders();
+    for (HeaderVec::const_iterator i = h.begin();
+         i != h.end(); i++) {
+        ne_add_request_header(_req, i->first.c_str(), i->second.c_str());
+      }
 }
 
 
+static std::string extract_bucket(const Uri & uri){
+    const std::string & hostname = uri.getHost();
+    std::string::const_iterator it = std::find(hostname.begin(), hostname.end(),'.');
+    return std::string(hostname.begin(), it);
+}
+
 void NEONRequest::configureS3params(){
     struct tm utc_current;
     time_t t = time(NULL);
@@ -232,7 +293,7 @@ void NEONRequest::configureS3params(){
        << "\n"          // TODO : implement Content-type and md5 parser
        << "\n"
        << date << "\n"
-       << _current.getPath();
+       << '/' << extract_bucket(*_current)  << _current->getPath();
     addHeaderField("Authorization", getAwsAuthorizationField(ss.str(), params.getAwsAutorizationKeys().first, params.getAwsAutorizationKeys().second));
 }
 
@@ -246,7 +307,8 @@ int NEONRequest::processRedirection(int neonCode, DavixError **err){
             neon_to_davix_code(neonCode, _neon_sess->get_ne_sess(), davix_scope_http_request(),err);
             return -1;
         }
-       _last_read=  ne_discard_response(_req);              // Get a valid redirection, drop request content
+        ne_discard_response(_req);              // Get a valid redirection, drop request content
+        _last_read =0;
         end_status = ne_end_request(_req);      // submit the redirection
         if(redirect_request(err) <0){           // accept redirection
             return -1;
@@ -260,26 +322,7 @@ int NEONRequest::processRedirection(int neonCode, DavixError **err){
 }
 
 
-int NEONRequest::startRequest(DavixError **err){
-    if( _cache_info.get() != NULL
-            && httpCacheTokenIsValid(_orig, *(_cache_info.get()))
-            && params.getTransparentRedirectionSupport() == true){
-        // Try to connect directly to the cached redirection, if exist
-        DavixError * tmp_err=NULL;
-        _current = _cache_info->getCachedRedirection();
-        DAVIX_TRACE("Try to use cached redirection %s", _current.getString().c_str());
-        if( create_req(&tmp_err) >= 0){
-            if(negotiate_request(&tmp_err) >= 0){
-                if(httpcodeIsValid(getRequestCode())){
-                    DAVIX_TRACE("use of %s with success", _current.getString().c_str());
-                    return 0;
-                }
-            }
-        }
-        DavixError::clearError(&tmp_err);
-        endRequest(NULL);
-        _current = _orig;
-    }
+int NEONRequest::startRequest(DavixError **err){        
     if( create_req(err) < 0)
             return -1;
     return negotiate_request(err);
@@ -289,7 +332,7 @@ int NEONRequest::negotiate_request(DavixError** err){
 
     const int n_limit = 10;
     int code, status, end_status = NE_RETRY;
-    int n =0;
+    _last_read = -1;
 
     DAVIX_DEBUG(" ->   Davix negociate request ... ");
     if(req_started){
@@ -298,29 +341,36 @@ int NEONRequest::negotiate_request(DavixError** err){
         return -1;
     }
 
-    req_started = req_running= true;
+    req_started = req_running = true;
 
-    while(end_status == NE_RETRY && n < n_limit){
+    while(end_status == NE_RETRY && _number_try < n_limit){
         DAVIX_DEBUG(" ->   NEON start internal request ... ");
 
         if( (status = ne_begin_request(_req)) != NE_OK && status != NE_REDIRECT){
-            if( status == NE_ERROR && strstr(ne_get_error(_neon_sess->get_ne_sess()), "Could not") != NULL ){ // bugfix against neon keepalive problem
-               DAVIX_DEBUG("Connexion close, retry...");
-               n++;
-               continue;
+            if( status == NE_ERROR ){ // bugfix against neon keepalive problem, protection against buggy servers
+                if(strstr(ne_get_error(_neon_sess->get_ne_sess()), "Could not") != NULL
+                   || strstr(ne_get_error(_neon_sess->get_ne_sess()), "Connection reset by peer") != NULL){
+                   DAVIX_DEBUG("KeepAlive Server problem detected, retry");
+                   _number_try++;
+                   redirect_cleanup();
+                   return startRequest(err);
+                }
             }
 
             req_started= req_running == false;
             neon_to_davix_code(status, _neon_sess->get_ne_sess(), davix_scope_http_request(),err);
             neonrequest_eradicate_session(*_neon_sess, err);
+            ContextExplorer::SessionFactoryFromContext(_c).redirectionClean(_request_type, *_orig);
             DAVIX_DEBUG(" Davix negociate request ... <-");
             return -1;
         }
 
+        _last_read =1;
         code = getRequestCode();
         switch(code){
             case 301:
             case 302:
+            case 303:
             case 307:
                 if( (end_status = processRedirection(status, err)) <0){
                    DAVIX_DEBUG(" Davix negociate request ... <-");
@@ -328,7 +378,8 @@ int NEONRequest::negotiate_request(DavixError** err){
                 }
                 break;
             case 401: // authentification requested, do retry
-                _last_read= ne_discard_response(_req);
+                ne_discard_response(_req);
+                _last_read =0;
                 end_status = ne_end_request(_req);
 
                 if( end_status != NE_RETRY){
@@ -340,19 +391,38 @@ int NEONRequest::negotiate_request(DavixError** err){
                     }else{
                         neon_to_davix_code(status, _neon_sess->get_ne_sess(), davix_scope_http_request(),err);
                     }
+                    _number_try++;
+                    if (redirect_cleanup()){
+                        DavixError::clearError(err);
+                        endRequest(NULL);
+                        return startRequest(err);
+                    }
+
                     return -1;
                 }
                 DAVIX_DEBUG(" ->   NEON receive %d code, %d .... request again ... ", code, end_status);
                 break;
+            // dCache token redirection
+            case 501:
+            // cleanup redirection
+            _number_try++;
+            if( redirect_cleanup()){
+                // recursive call, restart request
+                endRequest(NULL);
+                return startRequest(err);
+            }
+
+            return -1;
+
             default:
                 end_status = 0;
                 break;
 
         }
-        n++;
+        _number_try++;
     }
 
-    if(n >= n_limit){
+    if(_number_try >= n_limit){
         DavixError::setupError(err,davix_scope_http_request(),StatusCode::AuthentificationError,
                                "Maximum number of retrial reached.");
         return -2;
@@ -361,7 +431,24 @@ int NEONRequest::negotiate_request(DavixError** err){
     return 0;
 }
 
+
+bool NEONRequest::redirect_cleanup(){
+    // cleanup redirection
+    ContextExplorer::SessionFactoryFromContext(_c).redirectionClean(_request_type, *_orig);
+    // disable recycling
+    // server supporting broken pipelining will trigger if reused
+    _neon_sess->disable_session_reuse();
+    // cancel redirect, maybe outdated ? retry
+    if(_current != _orig){
+        DAVIX_DEBUG(" ->  Auth problem after redirect: cancel redirect and try again");
+        _current = _orig;
+        return true;
+    }
+    return false;
+}
+
 int NEONRequest::redirect_request(DavixError **err){
+    boost::shared_ptr<Uri> old_uri;
     const ne_uri * new_uri = ne_redirect_location(_neon_sess->get_ne_sess());
     if(!new_uri){
         DavixError::setupError(err, davix_scope_http_request(), StatusCode::UriParsingError, "Impossible to get the new redirected destination");
@@ -369,12 +456,14 @@ int NEONRequest::redirect_request(DavixError **err){
     }
 
     char* dst_uri = ne_uri_unparse(new_uri);
-    DAVIX_DEBUG("redirection from %s://%s/%s to %s", ne_get_scheme(_neon_sess->get_ne_sess()),
-                      ne_get_server_hostport(_neon_sess->get_ne_sess()), _current.getPathAndQuery().c_str(), dst_uri);
+    DAVIX_DEBUG("redirection from %s to %s", _current->getString().c_str(), dst_uri);
 
     // setup new path & session target
-    _current= Uri(dst_uri);
+    old_uri = _current;
+    _current= boost::shared_ptr<Uri>(new Uri(dst_uri));
     ne_free(dst_uri);
+    ContextExplorer::SessionFactoryFromContext(_c).addRedirection(_request_type, *old_uri, _current);
+
 
     // recycle old request and session
     free_request();
@@ -391,8 +480,9 @@ int NEONRequest::redirect_request(DavixError **err){
 }
 
 int NEONRequest::executeRequest(DavixError** err){
-    dav_ssize_t read_status=1;
+    dav_ssize_t read_status=1, total_read=0;
     _last_request_flag =0;
+    _vec.clear();
 
     DAVIX_DEBUG(" -> NEON start synchronous  request... ");
     if( startRequest(err) < 0){
@@ -400,26 +490,32 @@ int NEONRequest::executeRequest(DavixError** err){
     }
 
     if(getAnswerSize() > 0)
-        _vec.reserve(getAnswerSize());
+        _vec.reserve(std::min<size_t>(getAnswerSize(), 4194304));
 
     while(read_status > 0){
         DAVIX_DEBUG(" -> NEON Read data flow... ");
         size_t s = _vec.size();
         _vec.resize(s + NEON_BUFFER_SIZE);
         read_status= readBlock(&(_vec[s]), NEON_BUFFER_SIZE, err);
-        if( read_status >= 0 && read_status != NEON_BUFFER_SIZE){
-           _vec.resize(s +  read_status);
+        if( read_status >= 0){
+            if(read_status != NEON_BUFFER_SIZE){
+                _vec.resize(s +  read_status);
+            }
+           total_read += read_status;
         }
 
     }
-    // push a last NULL char for safety
-    _vec.push_back('\0');
 
     if(read_status < 0){
         if(err && *err == NULL)
             neon_to_davix_code(read_status, _neon_sess->get_ne_sess(), davix_scope_http_request(), err);
         return -1;
     }
+    _vec.push_back('\0');
+
+    if(_ans_size < 0){
+        _ans_size = total_read;
+    }
 
    if( endRequest(err) < 0){
        return -1;
@@ -452,25 +548,32 @@ dav_ssize_t NEONRequest::readBlock(char* buffer, dav_size_t max_size, DavixError
 
     // take from line buffer
     if(_vec_line.size() > 0){
-       if( _vec_line.size() > max_size){
-        stdgnu::copy_n(_vec_line.begin(), max_size, buffer);
+       if( _vec_line.size() >= max_size){
+        std::copy(_vec_line.begin(), _vec_line.begin() + max_size, buffer);
         _vec_line.erase(_vec_line.begin(), _vec_line.begin() + max_size);
+        DAVIX_TRACE("NEONRequest::readBlock read %ld bytes (from buffer)", max_size);
         return max_size;
        }else{
            const dav_ssize_t n_bytes = _vec_line.size();
            std::copy(_vec_line.begin(), _vec_line.end(), buffer);
            _vec_line.clear();
            read_status = readBlock(buffer + n_bytes, max_size -n_bytes, err);
-           return (read_status >= 0)?(read_status+n_bytes):(-1);
+           const dav_ssize_t ret_value = (read_status >= 0)?(read_status+n_bytes):(-1);
+           DAVIX_TRACE("NEONRequest::readBlock read %ld bytes(from partially)", ret_value);
+           return ret_value;
        }
     }
 
+    if(_last_read ==0)
+        return 0;
+
     _last_read= read_status= ne_read_response_block(_req, buffer, max_size );
     if(read_status <0){
        DavixError::setupError(err, davix_scope_http_request(), StatusCode::ConnectionProblem, "Invalid Read in request");
        req_running = false;
        return -1;
     }
+    DAVIX_TRACE("NEONRequest::readBlock read %ld bytes", read_status);
     return read_status;
 }
 
@@ -519,7 +622,7 @@ dav_ssize_t NEONRequest::readLine(char* buffer, dav_size_t max_size, DavixError*
        std::vector<char>::iterator it;
        it = std::find(_vec_line.begin(), _vec_line.end(), '\n');
 
-       if( it  < _vec_line.end()){
+       if( it  != _vec_line.end()){
            it ++;
            const dav_ssize_t read_size = it - _vec_line.begin();
            std::copy(_vec_line.begin(), it, buffer);
@@ -578,8 +681,11 @@ int NEONRequest::endRequest(DavixError** err){
     int status;
     if(_req  && req_running == true){
         req_running = false;
-        if(_last_read > 0) // if read content, discard it
+        if(_last_read > 0){ // if read content, discard it
+            DAVIX_TRACE("Discard remaining content....");
             ne_discard_response(_req);
+            _last_read =0;
+        }
         if( (status = ne_end_request(_req)) != NE_OK){
             DavixError* tmp_err=NULL;
             if(_neon_sess.get() != NULL)
@@ -619,7 +725,7 @@ dav_ssize_t NEONRequest::getAnswerSizeFromHeaders() const{
        DAVIX_TRACE("Bad server answer: %s Invalid, impossible to determine answer size", ans_header_content_length.c_str());
        size = -1;
     }
-    return (dav_ssize_t) size;
+    return static_cast<dav_ssize_t>(size);
 }
 
 dav_ssize_t NEONRequest::getAnswerSize() const{
@@ -640,6 +746,18 @@ bool NEONRequest::getAnswerHeader(const std::string &header_name, std::string &v
 }
 
 
+size_t NEONRequest::getAnswerHeaders( HeaderVec & vec_headers) const{
+    if(_req){
+        void * handle = NULL;
+        const char* name=NULL, *value=NULL;
+        while( (handle = ne_response_header_iterate(_req, handle, &name, &value)) != NULL){
+            vec_headers.push_back(std::pair<std::string, std::string>(name, value));
+        }
+    }
+    return vec_headers.size();
+}
+
+
 
 
 void NEONRequest::setRequestBody(const std::string & body){
@@ -670,11 +788,6 @@ void NEONRequest::setRequestBody(HttpBodyProvider provider, dav_size_t len, void
     _content_provider.udata    = udata;
 }
 
-HttpCacheToken* NEONRequest::extractCacheToken() const {
-    if(_last_request_flag ==0)
-        return NULL;
-    return HttpCacheTokenAccessor::createCacheToken(_orig, _current);
-}
 
 void NEONRequest::free_request(){
     if(_req != NULL){
@@ -683,4 +796,30 @@ void NEONRequest::free_request(){
     }
 }
 
+void NEONRequest::neon_hook_pre_send(ne_request *r, void *userdata,
+                   ne_buffer *header){
+    NEONRequest* req = (NEONRequest*) userdata;
+    RequestPreSendHook hook = req->_c.getHook<RequestPreSendHook>();
+    if(hook){
+        std::string header_line(header->data, (header->used)-1);
+        hook(req->_h, header_line);
+    }
+}
+
+void NEONRequest::neon_hook_pre_rec(ne_request *r, void *userdata,
+                                    const ne_status *status){
+    NEONRequest* req = (NEONRequest*) userdata;
+    RequestPreReceHook hook = req->_c.getHook<RequestPreReceHook>();
+    if(hook){
+        std::ostringstream header_line;
+        HeaderVec headers;
+        req->getAnswerHeaders(headers);
+        header_line << "HTTP/"<< status->major_version << '.' << status->minor_version
+                    << ' ' << status->code << ' ' << status->reason_phrase << '\n';
+
+        hook(req->_h, header_line.str(), headers, status->code);
+    }
+}
+
+
 } // namespace Davix
