@@ -1,5 +1,26 @@
-#include <config.h>
-#include <davix_types.h>
+/*
+ * This File is part of Davix, The IO library for HTTP based protocols
+ * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+*/
+
+#include <davix_internal.hpp>
+#include <utils/davix_logger_internal.hpp>
+#include <utils/davix_types.hpp>
 #include <status/davixstatusrequest.hpp>
 
 
@@ -39,9 +60,8 @@ DavixError::DavixError(const DavixError & e) :
 }
 
 DavixError & DavixError::operator =(const DavixError & e){
-    if(d_ptr)
-        delete d_ptr;
-    d_ptr = new DavixErrorInternal(*(e.d_ptr));
+    DavixError tmp(e);
+    this->swap(tmp);
     return *this;
 }
 
@@ -84,6 +104,7 @@ void DavixError::setErrScope(const std::string &scope){
 void DavixError::setupError(DavixError **err, const std::string &scope, StatusCode::Code errCode, const std::string &errMsg){
     if(err){
         if(*err){
+            DAVIX_DEBUG("Error Override of previous DavixError, BUG !");
             // error msg
         }
         *err = new DavixError(scope, errCode, errMsg);
@@ -96,10 +117,13 @@ void DavixError::propagateError(DavixError **newErr, DavixError *oldErr){
 }
 
 void DavixError::propagatePrefixedError(DavixError **newErr, DavixError *oldErr, const std::string &prefix){
+    if(oldErr== NULL)
+        return;
+
     if(newErr){
         if(*newErr != NULL){
             std::cerr << "***ERROR*** in propagateError, *newErr is not NULL impossible to overwrite ... "
-                     " old error wass" << ((oldErr)?(oldErr->getErrMsg()):"<NULL>") << std::endl;
+                     " old error was" << oldErr->getErrMsg()<< std::endl;
         }else{
             *newErr = oldErr;
             if(*newErr && prefix.empty() == false){
@@ -111,6 +135,10 @@ void DavixError::propagatePrefixedError(DavixError **newErr, DavixError *oldErr,
 
 }
 
+void DavixError::swap(DavixError &err){
+    std::swap(d_ptr, err.d_ptr);
+}
+
 void DavixError::clearError(DavixError **err){
     if(err && *err){
         delete *err;
@@ -156,6 +184,64 @@ void davix_errno_to_davix_error(int errcode, const std::string & scope, const st
     DavixError::setupError(newErr, scope, c, msg_final);
 }
 
+
+// extension
+struct DavixException::DavixExceptionIntern{
+
+};
+
+
+DavixException::DavixException(const std::string &scope, StatusCode::Code c, const std::string &msg) throw() :
+    std::exception(),
+    e(scope, c, msg),
+    d_ptr(NULL){
+
+}
+
+DavixException::DavixException(const DavixException &orig) throw() : e(orig.e), d_ptr(orig.d_ptr)
+{
+
+}
+
+DavixException::DavixException(DavixError **err) :
+std::exception(),
+  e( "Davix::Error", StatusCode::UnknowError, "Error, no valid DavixError triggered"),
+d_ptr(NULL){
+    if(err != NULL && *err != NULL){
+        e.swap(**err);
+        DavixError::clearError(err);
+    }
+}
+
+
+void checkDavixError(DavixError **err){
+    if(err && *err){
+        throw DavixException(err);
+    }
+}
+
+
+DavixException::~DavixException() throw(){
+    delete d_ptr;
+}
+
+const char* DavixException::what() const throw() {
+    return e.getErrMsg().c_str();
+}
+
+StatusCode::Code DavixException::code() const throw(){
+    return e.getStatus();
+}
+
+const char* DavixException::scope() const throw(){
+    return e.getErrScope().c_str();
+}
+
+void DavixException::toDavixError(DavixError **err){
+    DavixError::propagateError(err, new DavixError(this->e));
+}
+
+
 std::string davix_scope_stat_str(){
     return "Davix::Posix::stat";
 }
@@ -172,6 +258,10 @@ std::string davix_scope_http_request(){
     return "Davix::HttpRequest";
 }
 
+std::string davix_scope_meta(){
+    return "Davix::MetaDataOps";
+}
+
 std::string davix_scope_xml_parser(){
     return "Davix::XMLParser";
 }
